/*---------------------------------------------------------------.
| Displays info on QFAX file					 |
| 								 |
| qtest filename						 |
| 								 |
| (ex qtest;'file_fax' for QDOS)				 |
| 								 |
| cc -o qtest -I ~/develop/Fax/lib -L ~/develop/Fax/lib/ qtest.c |
| -lqfax							 |
`---------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <sys/times.h>

#include "qheader.h"
#include "qfaxparse.h"

#define VR_NORMAL	0
#define VR_FINE		1

#define DF_1DHUFFMAN 	0
#define DF_2DMREAD	1
#define DF_2DUNCOMP	2

short llim = 3;

char *vr_string (short vr)
{
    switch (vr)
    {
	case VR_NORMAL:
	    return ("normal");
	case VR_FINE:
	    return ("fine");
	default:
	    return ("unknown");
    }
}

char *df_string (short df)
{
    switch (df)
    {
	case DF_1DHUFFMAN:
	    return ("1D huffman");
	case DF_2DMREAD:
	    return ("2D read");
	case DF_2DUNCOMP:
	    return ("2D uncompressed");
	default:
	    return ("unknown");
    }
}


static char lzlead [256] =
{
    8,7,6,6,5,5,5,5,4,4,4,4,4,4,4,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
    2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
    0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
};

static char lztail [256] =
{
    8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
    5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
    6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
    5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
    7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
    5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
    6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
    5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,
};


/*
 * The reason for the static and rather weird way of doing this is
 * because this is usually in a different module
 */

static char *ztail = lztail;
static char *zlead = lzlead;

static  int zt,  nz;
static int nline;

void ZeroRTC (short flag)
{
    zt = nline = 0;

    if((flag & 1) == 0)
    {
        ztail = lztail;
        zlead = lzlead;
    }
    else
    {
        zlead = lztail;
        ztail = lzlead;
    }
}
    
int TestRTC (unsigned char c, int *zzz)
{
    int rval = 0;

    nz =  nz + zlead[c];
    if (c)
    {
	if ( nz > 10)
	{
            zt++;
            nline++;
	}
	else
	{
	    if (zt > llim)
	    {
		*zzz = zt;
		nline -= zt;
		rval = nline;
	    }
	    zt = 0;
	}
	nz = ztail[c];
    }
    return rval;
}

int EndRTC(int *zzz)
{
    *zzz = zt;
    return nline;
}

void ReadQfax(FILE *f1, QFAXHEADER *ph)
{

    int i,j;
    unsigned char buf[512];
    int c;
    int nb = 0;
    int nx;
    int nc = 64;
    int np = 0;
    
    ZeroRTC(ph->lsb);
    
    while((j = fread(buf, 1, 512, f1)) > 0)
    {
        for(i = 0; i < j  ; i++)
        {
	    nc++;
            c = buf[i];
	    nx = TestRTC(c, &nb);
	    if(nx && nb > llim-1)
	    {
		printf ("%3d: %5d %2d (%10d)\n", ++np, nx, nb, nc);
		ZeroRTC(ph->lsb);
		nx = 0;
	    }
        }
    }
    if(nx == 0)
    {
	nx = EndRTC(&nb);
	if(nb > 2)
	    printf ("%3d: %5d %2d (%10d)\n", ++np, nx, nb, nc);
    }

}

typedef union
{
    int x;
    char y[4];
} UV;

int main(int ac, char **av)
{
    FILE *fp1;
    QFAXHEADER h;
    UV hv;
#ifdef __linux__
    clock_t t1 =0,t2=0;
    struct tms tms;
#else
    time_t t1=0,t2=0;
#endif

    if(ac == 3)
    {
	llim = atoi(av[2]);
    }

    if((fp1 = fopen(av[1], "r")))
    {
        fread(&h, sizeof(h), 1, fp1);
#ifdef __linux__
	HeaderEndianise(&h);   /* Only required for little endian (!680x0) */
	hv.x = swaplong(h.version);
#else
	hv.x = h.version;
#endif
	if(*(hv.y+3) < '5') h.lsb = 0;
        if (h.title == QFAXTITLE)
        {
            puts   ("QFacts v0.1");
	    printf ("Version    : %4.4s\n", hv.y);
            printf ("FileName   : %s\n", av[1]);
            printf ("TSI/CSI    : %.23s\n", h.tsi);
            printf ("Date       : %s", ctime((long *)&h.stamp));
            printf ("Resolution : %s\n", vr_string(h.resolution));
	    printf ("Hint       : %d\n", h.hint);
	    printf ("Pages      : %d\n", h.pagemax);
            printf ("Coding     : %s (%csb)\n", df_string(h.coding), 
		    h.lsb ? 'l' : 'm');
#ifdef __linux__
	    t1 = times(&tms);
#else
	    t1 = time(NULL);
#endif
            ReadQfax(fp1, &h);
        }
        else
        {
            puts("Not QFAX");
        }
	if(t1)
	{
#ifdef __linux__
	    t2 = times(&tms) - t1;
	    printf("Time to parse %ld.%02lds\n", (t2/CLOCKS_PER_SEC), 
		   (t2 % CLOCKS_PER_SEC)); 
#else
	    t2 = time(NULL)-t1;
	    printf("Time to parse %ds\n", t2); 
#endif
	}
	fclose(fp1);	
    }
    else
    {
        perror("Input file");
    }
    return 0;
}




